package com.parksong.basics.aspect;

import com.alibaba.fastjson.JSON;
import com.mysql.jdbc.exceptions.jdbc4.MySQLIntegrityConstraintViolationException;
import com.parksong.basics.api.Limiting;
import com.parksong.basics.beans.*;
import com.parksong.basics.exception.AskException;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.exception.NotLoginException;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.basics.filter.Filter;
import com.parksong.basics.services.BasicsService;
import com.parksong.basics.util.constants.ReturnStatusCodeConstants;
import com.parksong.beans.express.ExpressDept;
import com.parksong.beans.user.User;
import com.parksong.services.log.LoggingEventService;
import com.parksong.services.user.UsersService;
import com.parksong.util.JwtUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.Objects;

/**
 * @ClassName ControllerAop
 * @Description controllerAop捕捉返回resultBean或者PageResultBean的类，
 * 用于统一打印日志、统一错误处理，service中不应该捕捉异常，统一交给Aop处理
 * @Author kael
 * @Date 2019/9/6 0006 上午 10:38
 * @Version 1.0
 **/
@Component
@Aspect
@Slf4j
public class ControllerAop {

    private final static boolean IS_LOG = true;

    @Autowired
    private UsersService usersService;

    @Autowired
    private LoggingEventService loggingEventService;

    @Autowired
    private BasicsService basicsService;

    @Before("@annotation(limiting)")
    public void before(JoinPoint joinPoint, Limiting limiting){
        //限流
        if(limiting != null && limiting.value()){
            Object[] args = joinPoint.getArgs();
            ApiFilter<?> filter = (ApiFilter<?>) args[0];
            StoreReqLimit storeReqLimit = basicsService.getStoreReqLimitByUserId(filter.getStoreId().toString());
            if(storeReqLimit.getIsLimitFlag()){
                throw new BusinessException("系统繁忙，请重试");
            }
        }
    }

    @Around("execution(public com.parksong.basics.beans.ResultBean *(..))")
    public Object aroundResultBean2(ProceedingJoinPoint pjp){
        ResultBean<?> result = getResultBean(pjp);
        return result;
    }

    private ResultBean<?> getResultBean(ProceedingJoinPoint pjp) {
        long startTime = System.currentTimeMillis();
        ResultBean<?> result;
        ApiFilter<?> filter = null;
        Object[] objs = pjp.getArgs();
        String requestMsg = "";
        try {
            requestMsg = JSON.toJSONString(objs);
            filter = this.loginGetUserInfo(objs);
            result = (ResultBean<?>) pjp.proceed();
            result.setRequestSign(filter.getData().getRequestSign());
        } catch (Throwable e) {
            result = handlerException(pjp, e);
        }
        this.addLog(filter, requestMsg, new Long((System.currentTimeMillis() - startTime)).intValue(), JSON.toJSONString(result), result.getCode());
        return result;
    }

    /**
     * 通过用户名和密码登录获取用户信息(应该使用验权系统进行权限验证，而不应该使用用户名和密码)
     * @param objs
     */
    private ApiFilter<?> loginGetUserInfo(Object[] objs) throws Exception {
        //判断是否登录
        if (null != objs && objs.length > 0) {
            ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

            HttpServletRequest request = Objects.requireNonNull(attributes).getRequest();
            log.info("【请求 URL】：{}", request.getRequestURL());
            log.info("【请求 IP】：{}", request.getRemoteAddr());
            log.info("【请求参数】：{}", JSON.toJSONString(objs));
            ApiFilter<?> filter = (ApiFilter<?>) objs[0];
            User user = this.getUserFromParam(filter.getUserId(),filter.getPassword());
            if(user==null){
                throw new NotLoginException("用户名或密码错误");
            }
            if(user.getStoreId().compareTo(filter.getStoreId()) != 0){
                throw new BusinessException("传入门店id和用户不相同");
            }
            if(filter.getData() == null){
                return this.getFilterApiFilter(filter, user);
            }
            filter.getData().setUser(user);
            filter.getData().setStoreId(filter.getStoreId());
            log.info("【请求用户】：{}，【请求门店】：{}",filter.getUserId(),filter.getStoreId());
            return filter;
        }else{
            log.info("【请求参数】：null");
            throw new BusinessException("传入参数不能为空");
        }

    }

    /**
     * 如果传入data数据为空则重构apiFilter对象
     * @param filter
     * @param user
     * @return
     */
    private ApiFilter<Filter> getFilterApiFilter(ApiFilter<?> filter, User user) {
        ApiFilter<Filter> apiFilter = new ApiFilter<Filter>();
        Filter filter1 = new Filter();
        filter1.setStoreId(filter.getStoreId());
        filter1.setUser(user);
        apiFilter.setData(filter1);
        apiFilter.setUserId(filter.getUserId());
        apiFilter.setAppVersion(filter.getAppVersion());
        apiFilter.setStoreId(filter.getStoreId());
        apiFilter.setPassword(filter.getPassword());
        return apiFilter;
    }

    /**
     * 异常处理，有别的异常新增异常类，然后加上异常判断
     * @param pjp
     * @param e
     * @return
     */
    private ResultBean<?> handlerException(ProceedingJoinPoint pjp, Throwable e) {
        ResultBean<?> result = new ResultBean();

        if(e instanceof MySQLIntegrityConstraintViolationException){
            result.setMsg("数据重复");
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof NotLoginException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.NO_LOGIN);
        } else if(e instanceof BusinessException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof AskException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.ASK);
        }  else if(e instanceof NullPointerException){
            result.setMsg("非法入参");
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else if(e instanceof Exception){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else {
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        }
        e.printStackTrace();
        return result;
    }

    /************************分页aop****************************/

    @Around("execution(public com.parksong.basics.beans.PageResultBean *(..))")
    public Object aroundPageResultBean(ProceedingJoinPoint pjp){
        PageResultBean<?> result = getPageResultBean(pjp);
        return result;
    }

    private PageResultBean<?> getPageResultBean(ProceedingJoinPoint pjp) {
        long startTime = System.currentTimeMillis();
        PageResultBean<?> result;
        Object[] objs = pjp.getArgs();
        ApiFilter<?> filter = null;
        String requestMsg = "";
        try {
            requestMsg = JSON.toJSONString(objs);
            filter = this.loginGetUserInfo(objs);
            //分页初始化数据
            filter.getData().init();
            result = (PageResultBean<?>) pjp.proceed();
            //分页返回当前页和查询条数
            if (filter != null) {
                result.setCurrentPage(filter.getData().getCurrentPage());
                result.setPageSize(filter.getData().getPageSize());
                result.setRequestSign(filter.getData().getRequestSign());
            }
        } catch (Throwable e) {
            result = handlerExceptionPageResultBean(pjp, e);
        }
        this.addLog(filter, requestMsg, new Long((System.currentTimeMillis() - startTime)).intValue(), JSON.toJSONString(result), result.getCode());
        return result;
    }

    private void addLog(ApiFilter<?> filter, String requestMsg, Integer exTime, String responseMsg, int code) {
        log.info("【返回值】：{}",  responseMsg);
        log.info("【请求耗时】：{}毫秒", exTime);
        try {
            //是否启用日志，之后可以使用数据库配置的方式
            if(!IS_LOG||code==0){
                return;
            }
            if (filter == null) {
                loggingEventService.addLog(requestMsg, responseMsg, code, exTime);
            } else {
                loggingEventService.addLog(requestMsg, responseMsg, filter.getUserId(), filter.getStoreId(), code, "1", exTime);
            }
        }catch (Exception e){
            log.error(e.getMessage());
            e.printStackTrace();
        }

    }

    /**
     * 异常处理，有别的异常新增异常类，然后加上异常判断
     * @param pjp
     * @param e
     * @return
     */
    private PageResultBean<?> handlerExceptionPageResultBean(ProceedingJoinPoint pjp, Throwable e) {
        PageResultBean<?> result = new PageResultBean();

        if(e instanceof MySQLIntegrityConstraintViolationException){
            result.setMsg("数据重复");
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof NotLoginException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.NO_LOGIN);
        } else if(e instanceof BusinessException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof AskException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.ASK);
        } else if(e instanceof NullPointerException){
            result.setMsg("非法入参");
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else if(e instanceof Exception){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else {
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        }
        e.printStackTrace();
        return result;
    }

    /**
     * 修改原方法从缓存里取数据
     * @param username
     * @param password
     * @return
     * @throws NotLoginException
     */
    public User getUserFromParam(String username, String password) throws NotLoginException{
        if(!Strings.isBlank(username) && !Strings.isBlank(password) ){
            User user = usersService.getUserById(username);
            if(null == user){
                throw new NotLoginException("查询不到用户名为"+username+"的用户");
            }
            if(password.equals(user.getPassword())){
                return user;
            }
        }
        throw new NotLoginException("查询不到用户名为"+username+"的用户");
    }

    /************************微信小程序aop****************************/
    @Around("execution(public com.parksong.basics.beans.ResultNo *(..))")
    public Object aroundResultNo(ProceedingJoinPoint pjp){
        ResultNo<?> resultNo = getResultNo(pjp);
        return resultNo;
    }

    private ResultNo<?> getResultNo(ProceedingJoinPoint pjp) {
        long startTime = System.currentTimeMillis();
        ResultNo<?> resultNo;
        ApiFilter<?> filter = new ApiFilter<>();
        Object[] objects = pjp.getArgs();
        String requestMsg = "";
        try {
            requestMsg = JSON.toJSONString(objects);
            filter = this.dealwithToken(objects);
            resultNo = (ResultNo<?>) pjp.proceed();
        } catch (Throwable e) {
            resultNo = handlerExceptionResultNo(pjp, e);
        }
        long endTime = System.currentTimeMillis();
        this.addLog(filter, requestMsg, (int) (endTime - startTime), JSON.toJSONString(resultNo), resultNo.getCode());
        return resultNo;
    }

    private ApiFilter<?> dealwithToken(Object[] objects) throws Exception {
        ApiFilter<?> filter = (ApiFilter<?>) objects[0];
        ExpressDept expressDept = new ExpressDept();
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = ((ServletRequestAttributes)requestAttributes).getRequest();
        String token = request.getHeader("token");
        log.info("【请求 URL】：{}", request.getRequestURL());
        log.info("【请求 IP】：{}", request.getRemoteAddr());
        log.info("【请求参数】：{}", JSON.toJSONString(objects));
        if ((Strings.isNotBlank(token))){
            Map<String, String> map = JwtUtils.verifyToken(token);
            expressDept.setId(Integer.parseInt(map.get("uid")));
            expressDept.setStoreId(Integer.parseInt(map.get("storeid")));
            expressDept.setExpId(Long.valueOf(map.get("expid")));
            expressDept.setOwnerName(map.get("uname"));
            expressDept.setOwnerPhone(map.get("uphone"));
        }else {
            if (null != objects && objects.length > 0) {
                if (Strings.isBlank(filter.getAccount()) || Strings.isBlank(filter.getPassword())){
                    throw new NotLoginException("用户名或者密码为空");
                }
                filter.setAccount(filter.getAccount());
                filter.setPassword(filter.getPassword());
                log.info("【请求用户】：{}",expressDept.getOwnerPhone());
                return filter;
            }else{
                log.info("【请求参数】：null");
                throw new BusinessException("传入参数不能为空");
            }
        }
        if (filter.getData() == null){
            ApiFilter<Filter> apiFilter = new ApiFilter<Filter>();
            Filter filter1 = new Filter();
            filter1.setExpressDept(expressDept);
            apiFilter.setData(filter1);
            return apiFilter;
        }else {
            filter.getData().setExpressDept(expressDept);
            filter.getData().setStoreId(expressDept.getStoreId());
        }
        return filter;
    }

    private ResultNo<?> handlerExceptionResultNo(ProceedingJoinPoint pjp, Throwable e) {
        ResultNo<?> result = new ResultNo<>();
        if(e instanceof MySQLIntegrityConstraintViolationException){
            result.setMsg("数据重复");
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof NotLoginException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.NO_LOGIN);
        } else if(e instanceof BusinessException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof AskException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.ASK);
        }  else if(e instanceof NullPointerException){
            result.setMsg("非法入参");
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else if(e instanceof Exception){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else {
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        }
        e.printStackTrace();
        return result;
    }

    /************************微信小程序分页aop****************************/
    @Around("execution(public com.parksong.basics.beans.PageResultNo *(..))")
    public Object aroundPageResultNo(ProceedingJoinPoint pjp){
        PageResultNo<?> result = getPageResultNo(pjp);
        return result;
    }

    private PageResultNo<?> getPageResultNo(ProceedingJoinPoint pjp) {
        long startTime = System.currentTimeMillis();
        PageResultNo<?> result;
        Object[] objs = pjp.getArgs();
        ApiFilter<?> filter = null;
        String requestMsg = "";
        try {
            requestMsg = JSON.toJSONString(objs);
            filter = this.dealwithToken(objs);
            //分页初始化数据
            filter.getData().init();
            result = (PageResultNo<?>) pjp.proceed();
            //分页返回当前页和查询条数
            if (filter != null) {
                result.setCurrentPage(filter.getData().getCurrentPage());
                result.setPageSize(filter.getData().getPageSize());
                result.setRequestSign(filter.getData().getRequestSign());
            }
        } catch (Throwable e) {
            result = handlerExceptionPageResultNo(pjp, e);
        }
        this.addLog(filter, requestMsg, new Long((System.currentTimeMillis() - startTime)).intValue(), JSON.toJSONString(result), result.getCode());
        return result;
    }

    private PageResultNo<?> handlerExceptionPageResultNo(ProceedingJoinPoint pjp, Throwable e) {
        PageResultNo<?> result = new PageResultNo<>();

        if(e instanceof MySQLIntegrityConstraintViolationException){
            result.setMsg("数据重复");
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof NotLoginException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.NO_LOGIN);
        } else if(e instanceof BusinessException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.FAIL);
        } else if(e instanceof AskException){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.ASK);
        } else if(e instanceof NullPointerException){
            result.setMsg("非法入参");
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else if(e instanceof Exception){
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        } else {
            result.setMsg(e.getMessage());
            result.setCode(ReturnStatusCodeConstants.Unknown);
        }
        e.printStackTrace();
        return result;
    }
}
