package com.parksong.basics.util.configurations;

import lombok.Data;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.amqp.RabbitProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

/**
 * @ClassName RabbitConfig
 * @Description rabbit参数
 * @Author kael
 * @Date 2019/9/27 0027 上午 11:28
 * @Version 1.0
 **/
@Data
@Component
@ConfigurationProperties(prefix = "spring.rabbitmq")
@Configuration
public class RabbitConfig {
    //实例Id 从阿里云AMQP控制台获取
    private static final String INSTANCE_ID = "amqp-cn-n6w24h52001y";
    @Autowired
    private RabbitProperties rabbitProperties;

    /**
     * 短信发送消息
     */
    private String smsQueueName;
    private String smsRoutingKey;
    private String smsExchangeName;

    /**
     * 快递发送第三方信息
     */
    private String expressQueue;
    private String expressExchange;
    private String expressKey;

    /**
     * 微信签收提醒
     */
    private String wechatQueue;
    private String wechatExahcange;
    private String wechatKey;

    /**
     * 同步手机号
     */
    private String phoneQueueName;
    private String phoneExchangeName;
    private String phoneRoutingKey;


    @Bean
    public ConnectionFactory getConnectionFactory() {
        com.rabbitmq.client.ConnectionFactory rabbitConnectionFactory = new com.rabbitmq.client.ConnectionFactory();
        rabbitConnectionFactory.setHost(rabbitProperties.getHost());
        rabbitConnectionFactory.setPort(rabbitProperties.getPort());
        rabbitConnectionFactory.setVirtualHost(rabbitProperties.getVirtualHost());

        AliyunCredentialsProvider credentialsProvider = new AliyunCredentialsProvider(rabbitProperties.getUsername(), rabbitProperties.getPassword(), INSTANCE_ID);
        rabbitConnectionFactory.setCredentialsProvider(credentialsProvider);
        rabbitConnectionFactory.setAutomaticRecoveryEnabled(true);
        rabbitConnectionFactory.setNetworkRecoveryInterval(5000);
        ConnectionFactory connectionFactory = new CachingConnectionFactory(rabbitConnectionFactory);
        ((CachingConnectionFactory) connectionFactory).setPublisherConfirms(rabbitProperties.isPublisherConfirms());
        ((CachingConnectionFactory) connectionFactory).setPublisherReturns(rabbitProperties.isPublisherReturns());
        return connectionFactory;
    }

    @Bean
    public RabbitAdmin rabbitAdmin(ConnectionFactory getConnectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(getConnectionFactory);
        //设置忽略声明异常
        rabbitAdmin.setIgnoreDeclarationExceptions(true);
        return rabbitAdmin;
    }

    // 1.定义快递队列
    @Bean
    public Queue expressQueue() {
        return new Queue(expressQueue, true);
    }
    // 2.定义交换机
    @Bean
    DirectExchange expressExchange() {
        return new DirectExchange(expressExchange, true, false);
    }
    // 3.队列与交换机绑定快递队列
    @Bean
    public Binding bindingExpressExchange(Queue expressQueue, DirectExchange expressExchange) {
        return BindingBuilder.bind(expressQueue).to(expressExchange).with(expressKey);
    }

    // 1.定义短信队列
    @Bean
    public Queue smsQueue() {
        return new Queue(smsQueueName, true);
    }
    // 2.定义短信交换机
    @Bean
    DirectExchange smsExchange() {
        return new DirectExchange(smsExchangeName, true, false);
    }
    // 3.短信队列与交换机绑定
    @Bean
    public Binding bindingSmsExchange(Queue smsQueue, DirectExchange smsExchange) {
        return BindingBuilder.bind(smsQueue).to(smsExchange).with(smsRoutingKey);
    }

    // 1.定义weChat队列
    @Bean
    public Queue weChatQueue() {
        return new Queue(wechatQueue, true);
    }
    // 2.定义交换机
    @Bean
    DirectExchange weChatExchange() {
        return new DirectExchange(wechatExahcange, true, false);
    }
    // 3.队列与交换机绑定weChat队列
    @Bean
    public Binding bindingWeChatExchange(Queue weChatQueue, DirectExchange weChatExchange) {
        return BindingBuilder.bind(weChatQueue).to(weChatExchange).with(wechatKey);
    }

    // 1.定义PhoneLib队列
    @Bean
    public Queue phoneQueue() {
        return new Queue(phoneQueueName, true);
    }
    // 2.定义交换机
    @Bean
    DirectExchange phoneExchange() {
        return new DirectExchange(phoneExchangeName, true, false);
    }
    // 3.队列与交换机绑定weChat队列
    @Bean
    public Binding phoneExchange(Queue phoneQueue, DirectExchange phoneExchange) {
        return BindingBuilder.bind(phoneQueue).to(phoneExchange).with(wechatKey);
    }
}
