package com.parksong.util.Http;


import com.alibaba.fastjson.JSON;
import lombok.Builder;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * @author taohaijun
 * @Description: OkHttpUtil
 * @date 2018/8/39:42
 */
@Slf4j
public class OkHttpUtil {
    public final static String GET = "GET";

    public final static String POST = "POST";

    public final static String PUT = "PUT";

    public final static String DELETE = "DELETE";

    public final static String PATCH = "PATCH";

    private final static String UTF8 = "UTF-8";

    private final static String GBK = "GBK";

    private final static String DEFAULT_CHARSET = UTF8;

    private final static String DEFAULT_METHOD = GET;

    private final static String DEFAULT_MEDIA_TYPE = "application/json";

    private final static boolean DEFAULT_LOG = false;

    /*private final static OkHttpClient client = new OkHttpClient.Builder().connectionPool(
            new ConnectionPool(1000, 30, TimeUnit.MINUTES)).readTimeout(20, TimeUnit.SECONDS).connectTimeout(
            20, TimeUnit.SECONDS).build();*/
    private final static OkHttpClient client = new OkHttpClient.Builder().connectionPool(
            new ConnectionPool(20000, 40, TimeUnit.SECONDS)).readTimeout(40, TimeUnit.SECONDS).connectTimeout(
            100, TimeUnit.SECONDS).build();

    /**
     * GET请求
     *
     * @param url URL地址
     * @return
     */
    public static String get(String url)
            throws Exception {
        return execute(OkHttp.builder().url(url).build());
    }

    /**
     * GET请求
     *
     * @param url URL地址
     * @return
     */
    public static String get(String url, String charset)
            throws Exception {
        return execute(OkHttp.builder().url(url).responseCharset(charset).build());
    }

    /**
     * 带查询参数的GET查询
     *
     * @param url      URL地址
     * @param queryMap 查询参数
     * @return
     */
    public static String get(String url, Map<String, String> queryMap)
            throws Exception {
        return execute(OkHttp.builder().url(url).queryMap(queryMap).build());
    }

    /**
     * 带查询参数的GET查询
     *
     * @param url      URL地址
     * @param queryMap 查询参数
     * @return
     */
    public static String get(String url, Map<String, String> queryMap, String charset)
            throws Exception {
        return execute(OkHttp.builder().url(url).queryMap(queryMap).responseCharset(charset).build());
    }

    /**
     * POST application/json
     *
     * @param url
     * @param obj
     * @return
     */
    public static String postJson(String url, Object obj)
            throws Exception {
        return execute(OkHttp.builder().url(url).method(POST).data(JSON.toJSONString(obj)).mediaType(
                "application/json").build());
    }

    /**
     * POST application/json
     *
     * @param url
     * @param json
     * @return
     */
    public static String postJson(String url, String json)
            throws Exception {
        return execute(OkHttp.builder().url(url).method(POST).data(json).mediaType(
                "application/json").build());
    }

    public static String postJson(String url,  Object obj, Long timeOut) throws Exception {
        OkHttpClient.Builder clientBuilder = new OkHttpClient.Builder();
        if(null != timeOut && timeOut > 0) {
            clientBuilder.connectTimeout(timeOut, TimeUnit.MILLISECONDS);
        }
        OkHttpClient okHttpClient = clientBuilder.build();
        // 设置请求类型
        String method = POST;
        String mediaType = String.format("%s;charset=%s", "application/json", UTF8);

        RequestBody requestBody = RequestBody.create(MediaType.parse(mediaType), JSON.toJSONString(obj));
        Request.Builder builder = new Request.Builder();
        builder.url(url);
        builder.method(method, requestBody);

        // 返回值
        String result = "";
        try {
            Response response = okHttpClient.newCall(builder.build()).execute();
            byte[] bytes = response.body().bytes();
            result = new String(bytes, UTF8);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return result;
    }


    private static String post(String url, String data, String mediaType, String charset)
            throws Exception {
        return execute(OkHttp.builder().url(url).method(POST).data(data).mediaType(mediaType).responseCharset(
                charset).build());
    }

    public static String posta(String url, String data, String mediaType)
            throws Exception {
        return execute(OkHttp.builder().url(url).method(POST).data(data).mediaType(mediaType).build());
    }

    /**
     * 通用执行方法
     */
    private static String execute(OkHttp okHttp)
            throws Exception {
        if (StringUtils.isEmpty(okHttp.requestCharset)) {
            okHttp.requestCharset = DEFAULT_CHARSET;
        }
        if (StringUtils.isEmpty(okHttp.responseCharset)) {
            okHttp.responseCharset = DEFAULT_CHARSET;
        }
        if (StringUtils.isEmpty(okHttp.method)) {
            okHttp.method = DEFAULT_METHOD;
        }
        if (StringUtils.isEmpty(okHttp.mediaType)) {
            okHttp.mediaType = DEFAULT_MEDIA_TYPE;
        }
        if (okHttp.requestLog) {// 记录请求日志
            log.info("[INFO]-requestLog" + okHttp.toString());
        }

        // 获取请求URL
        String url = okHttp.url;
        // 创建请求
        Request.Builder builder = new Request.Builder();
        if (MapUtils.isNotEmpty(okHttp.queryMap)) {
            String queryParams = okHttp.queryMap.entrySet().stream().map(
                    entry -> String.format("%s=%s", entry.getKey(), entry.getValue())).collect(
                    Collectors.joining("&"));
            url = String.format("%s%s%s", url, url.contains("?") ? "&" : "?", queryParams);
        }
        builder.url(url);

        // 设置请求头
        if (MapUtils.isNotEmpty(okHttp.headerMap)) {
            okHttp.headerMap.forEach(builder::addHeader);
        }

        // 设置请求类型
        String method = okHttp.method.toUpperCase();
        String mediaType = String.format("%s;charset=%s", okHttp.mediaType, okHttp.requestCharset);

        if (method.equals(GET)) {
            builder.get();
        } else if (ArrayUtils.contains(new String[]{POST, PUT, DELETE, PATCH}, method)) {
            RequestBody requestBody = RequestBody.create(MediaType.parse(mediaType), okHttp.data);
            builder.method(method, requestBody);
        } else {
            throw new Exception("未设置请求method");
        }

        // 返回值
        String result = "";
        int res = 0;
        try {
            Date begin = new Date();
            Response response = client.newCall(builder.build()).execute();
            byte[] bytes = response.body().bytes();
            Date end = new Date();
            res = Integer.valueOf(String.valueOf(end.getTime() - begin.getTime()));
            result = new String(bytes, okHttp.responseCharset);
            if (okHttp.responseLog) {// 记录返回日志
                log.info("[INFO]-responseLog" + result);
            }
        } catch (Exception e) {
            if (res > 40) {
                log.error("请求超时");
            }
            log.error(e.getMessage());
        }
        return result;
    }

    /**
     * 一个内部类
     *
     * @author gogym
     * @version 2018年7月30日
     * @see OkHttp
     * @since
     */

    @Builder
    @ToString(exclude = {"requestCharset", "responseCharset", "requestLog", "responseLog"})
    static class OkHttp {
        private String url;

        private String method = DEFAULT_METHOD;

        private String data;

        private String mediaType = DEFAULT_MEDIA_TYPE;

        private Map<String, String> queryMap;

        private Map<String, String> headerMap;

        private String requestCharset = DEFAULT_CHARSET;

        private boolean requestLog = DEFAULT_LOG;

        private String responseCharset = DEFAULT_CHARSET;

        private boolean responseLog = DEFAULT_LOG;
    }
}

